@@ -152,10 +152,12 @@ class GlossaryInfo:
     @staticmethod
     def from_json(json) -> "GlossaryInfo":
         """Create GlossaryInfo from the given API JSON object."""
-        # Workaround for bug in Python 3.6
+        # Workaround for bugs in strptime() in Python 3.6
         creation_time = json["creation_time"]
         if ":" == creation_time[-3:-2]:
             creation_time = creation_time[:-3] + creation_time[-2:]
+        if "Z" == creation_time[-1:]:
+            creation_time = creation_time[:-1] + "+0000"
 
         return GlossaryInfo(
             json["glossary_id"],
@@ -302,20 +304,56 @@ class Language:
         """Removes the regional variant from a language, e.g. EN-US gives EN"""
         return str(language).upper()[0:2]
 
-    @staticmethod
-    def check_supported_glossary_languages(source_lang: str, target_lang: str):
-        if f"{source_lang}>{target_lang}" not in [
-            "DE>EN",
-            "EN>DE",
-            "EN>ES",
-            "EN>FR",
-            "ES>EN",
-            "FR>EN",
-        ]:
-            raise DeepLException(
-                "Invalid source or target language, glossaries are supported "
-                "for the following language pairs: EN<->DE, EN<->FR, EN<->ES"
-            )
+    BULGARIAN = "bg"
+    CZECH = "cs"
+    DANISH = "da"
+    GERMAN = "de"
+    GREEK = "el"
+    ENGLISH = "en"  # Only usable as a source language
+    ENGLISH_BRITISH = "en-GB"  # Only usable as a target language
+    ENGLISH_AMERICAN = "en-US"  # Only usable as a target language
+    SPANISH = "es"
+    ESTONIAN = "et"
+    FINNISH = "fi"
+    FRENCH = "fr"
+    HUNGARIAN = "hu"
+    ITALIAN = "it"
+    JAPANESE = "ja"
+    LITHUANIAN = "lt"
+    LATVIAN = "lv"
+    DUTCH = "nl"
+    POLISH = "pl"
+    PORTUGUESE = "pt"  # Only usable as a source language
+    PORTUGUESE_BRAZILIAN = "pt-BR"  # Only usable as a target language
+    PORTUGUESE_EUROPEAN = "pt-PT"  # Only usable as a target language
+    ROMANIAN = "ro"
+    RUSSIAN = "ru"
+    SLOVAK = "sk"
+    SLOVENIAN = "sl"
+    SWEDISH = "sv"
+    CHINESE = "zh"
+
+
+class GlossaryLanguagePair:
+    """Information about a pair of languages supported for DeepL glossaries.
+
+    :param source_lang: The code of the source language.
+    :param target_lang: The code of the target language.
+    """
+
+    def __init__(self, source_lang: str, target_lang: str):
+        self._source_lang = source_lang
+        self._target_lang = target_lang
+
+    @property
+    def source_lang(self) -> str:
+        """Returns the code of the source language."""
+        return self._source_lang
+
+    @property
+    def target_lang(self) -> str:
+        """Returns the code of the target language."""
+        return self._target_lang
 
 
 class Formality(Enum):
@@ -358,8 +396,13 @@ class Translator:
     :param auth_key: Authentication key as found in your DeepL API account.
     :param server_url: (Optional) Base URL of DeepL API, can be overridden e.g.
         for testing purposes.
-    :param skip_language_check: (Optional) Set to True to override automatic
-        request of available languages.
+    :param proxy: (Optional) Proxy server URL string or dictionary containing
+        URL strings for the 'http' and 'https' keys. This is passed to the
+        underlying requests session, see the requests proxy documentation for
+        more information.
+    :param skip_language_check: Deprecated, and now has no effect as the
+        corresponding internal functionality has been removed. This parameter
+        will be removed in a future version.
 
     All functions may raise DeepLException or a subclass if a connection error
     occurs.
@@ -377,6 +420,7 @@ class Translator:
         auth_key: str,
         *,
         server_url: Optional[str] = None,
+        proxy: Union[Dict, str, None] = None,
         skip_language_check: bool = False,
     ):
         if not auth_key:
@@ -390,13 +434,9 @@ class Translator:
             )
 
         self._server_url = server_url
-        self._client = http_client.HttpClient()
+        self._client = http_client.HttpClient(proxy)
         self.headers = {"Authorization": f"DeepL-Auth-Key {auth_key}"}
 
-        self._skip_language_check = skip_language_check
-        self._source_languages_cached = None
-        self._target_languages_cached = None
-
     def __del__(self):
         self.close()
 
@@ -493,26 +533,10 @@ class Translator:
                 f"content: {content}."
             )
 
-    def _request_languages(self, target: bool) -> List[Language]:
-        """Internal function to make a /languages request and cache the result."""
-        data = {"type": "target"} if target else {}
-        status, content, json = self._api_call("v2/languages", data=data)
-
-        self._raise_for_status(status, content, json)
-
-        return [
-            Language(
-                language["language"],
-                language["name"],
-                language.get("supports_formality", None),
-            )
-            for language in json
-        ]
-
     def _check_valid_languages(
         self, source_lang: Optional[str], target_lang: str
     ):
-        """Internal function to check given languages match available languages."""
+        """Internal function to check given languages are valid."""
         if target_lang == "EN":
             raise DeepLException(
                 'target_lang="EN" is deprecated, please use "EN-GB" or "EN-US" instead.'
@@ -522,24 +546,6 @@ class Translator:
                 'target_lang="PT" is deprecated, please use "PT-PT" or "PT-BR" instead.'
             )
 
-        if self._skip_language_check:
-            return
-
-        if source_lang is not None and not any(
-            source_lang == lang.code for lang in self.get_source_languages()
-        ):
-            raise DeepLException(
-                f"source_lang ({source_lang}) must be one of the supported "
-                "language codes, or None for auto-detection"
-            )
-
-        if not any(
-            target_lang == lang.code for lang in self.get_target_languages()
-        ):
-            raise DeepLException(
-                f"target_lang ({target_lang}) must be one of the supported language codes"
-            )
-
     def _check_language_and_formality(
         self,
         source_lang: Union[str, Language, None],
@@ -564,10 +570,6 @@ class Translator:
                 raise ValueError(
                     "source_lang and target_lang must match glossary"
                 )
-        elif glossary is not None:
-            Language.check_supported_glossary_languages(
-                source_lang, Language.remove_regional_variant(target_lang)
-            )
 
         self._check_valid_languages(source_lang, target_lang)
 
@@ -742,6 +744,7 @@ class Translator:
         source_lang: Optional[str] = None,
         target_lang: str,
         formality: Union[str, Formality] = Formality.DEFAULT,
+        glossary: Union[str, GlossaryInfo, None] = None,
     ) -> None:
         """Upload document at given input path, translate it into the target
         language, and download result to given output path.
@@ -754,6 +757,8 @@ class Translator:
             "EN-US", "FR".
         :param formality: (Optional) Desired formality for translation, as Formality
             enum, "less" or "more".
+        :param glossary: (Optional) glossary or glossary ID to use for
+            translation. Must match specified source_lang and target_lang.
 
         :raises DocumentTranslationException: If an error occurs during translation,
             The exception includes information about the document request.
@@ -767,6 +772,7 @@ class Translator:
                         target_lang=target_lang,
                         source_lang=source_lang,
                         formality=formality,
+                        glossary=glossary,
                     )
                 except Exception as e:
                     out_file.close()
@@ -781,6 +787,7 @@ class Translator:
         source_lang: Optional[str] = None,
         target_lang: str,
         formality: Union[str, Formality] = Formality.DEFAULT,
+        glossary: Union[str, GlossaryInfo, None] = None,
     ) -> None:
         """Upload document, translate it into the target language, and download
         result.
@@ -795,6 +802,8 @@ class Translator:
             example "DE", "EN-US", "FR".
         :param formality: (Optional) Desired formality for translation, as
             Formality enum, "less" or "more".
+        :param glossary: (Optional) glossary or glossary ID to use for
+            translation. Must match specified source_lang and target_lang.
 
         :raises DocumentTranslationException: If an error occurs during
             translation, the exception includes the document handle.
@@ -805,12 +814,18 @@ class Translator:
             target_lang=target_lang,
             source_lang=source_lang,
             formality=formality,
+            glossary=glossary,
         )
 
         try:
             status = self.translate_document_get_status(handle)
             while status.ok and not status.done:
-                secs = (status.seconds_remaining or 0) / 2 + 1
+                secs = (status.seconds_remaining or 0) / 2.0 + 1.0
+                secs = max(1.0, min(secs, 60.0))
+                util.log_info(
+                    f"Rechecking document translation status "
+                    f"after sleeping for {secs:.2f} seconds."
+                )
                 time.sleep(secs)
                 status = self.translate_document_get_status(handle)
 
@@ -831,6 +846,7 @@ class Translator:
         source_lang: Optional[str] = None,
         target_lang: str,
         formality: Union[str, Formality] = Formality.DEFAULT,
+        glossary: Union[str, GlossaryInfo, None] = None,
     ) -> DocumentHandle:
         """Upload document to be translated and return handle associated with
         request.
@@ -844,11 +860,13 @@ class Translator:
             example "DE", "EN-US", "FR".
         :param formality: (Optional) Desired formality for translation, as
             Formality enum, "less" or "more".
+        :param glossary: (Optional) glossary or glossary ID to use for
+            translation. Must match specified source_lang and target_lang.
         :return: DocumentHandle with ID and key identifying document.
         """
 
         request_data = self._check_language_and_formality(
-            source_lang, target_lang, formality
+            source_lang, target_lang, formality, glossary
         )
 
         files = {"file": input_document}
@@ -922,20 +940,57 @@ class Translator:
             return response
 
     def get_source_languages(self, skip_cache=False) -> List[Language]:
-        """Request the list of available source languages."""
-        if self._source_languages_cached is None or skip_cache:
-            self._source_languages_cached = self._request_languages(
-                target=False
+        """Request the list of available source languages.
+
+        :param skip_cache: Deprecated, and now has no effect as the
+            corresponding internal functionality has been removed. This
+            parameter will be removed in a future version.
+        :return: List of supported source languages.
+        """
+        status, content, json = self._api_call("v2/languages")
+        self._raise_for_status(status, content, json)
+        return [
+            Language(
+                language["language"],
+                language["name"],
             )
-        return self._source_languages_cached
+            for language in json
+        ]
 
     def get_target_languages(self, skip_cache=False) -> List[Language]:
-        """Request the list of available target languages."""
-        if self._target_languages_cached is None or skip_cache:
-            self._target_languages_cached = self._request_languages(
-                target=True
+        """Request the list of available target languages.
+
+        :param skip_cache: Deprecated, and now has no effect as the
+            corresponding internal functionality has been removed. This
+            parameter will be removed in a future version.
+        :return: List of supported target languages.
+        """
+        data = {"type": "target"}
+        status, content, json = self._api_call("v2/languages", data=data)
+        self._raise_for_status(status, content, json)
+        return [
+            Language(
+                language["language"],
+                language["name"],
+                language.get("supports_formality", None),
             )
-        return self._target_languages_cached
+            for language in json
+        ]
+
+    def get_glossary_languages(self) -> List[GlossaryLanguagePair]:
+        """Request the list of language pairs supported for glossaries."""
+        status, content, json = self._api_call(
+            "v2/glossary-language-pairs", method="GET"
+        )
+
+        self._raise_for_status(status, content, json)
+
+        return [
+            GlossaryLanguagePair(
+                language_pair["source_lang"], language_pair["target_lang"]
+            )
+            for language_pair in json["supported_languages"]
+        ]
 
     def get_usage(self) -> Usage:
         """Requests the current API usage."""
@@ -956,9 +1011,10 @@ class Translator:
         languages, containing the entries in dictionary. The glossary may be
         used in the translate_text functions.
 
-        Only the following language pairs are supported: EN<->DE, EN<->FR, and
-        EN<->ES. A glossary with target language EN may be used to translate
-        texts into both EN-US and EN-GB.
+        Only certain language pairs are supported, currently the following:
+        EN<->DE, EN<->FR, and EN<->ES. The available language pairs can be
+        queried using get_glossary_languages(). A glossary with target language
+        EN may be used to translate texts into both EN-US and EN-GB.
 
         :param name: user-defined name to attach to glossary.
         :param source_lang: Language of source terms.
@@ -975,7 +1031,6 @@ class Translator:
         # glossaries are only supported for base language types
         target_lang = Language.remove_regional_variant(target_lang)
         source_lang = Language.remove_regional_variant(source_lang)
-        Language.check_supported_glossary_languages(source_lang, target_lang)
 
         if not name:
             raise ValueError("glossary name must not be empty")
